==> vee_1.9-0.tar.gz: vee-1.9/src/Bookmarks.py <==
# -*- coding: utf-8 -*-
import os, marshal, time, base64
import Conf, GConf, UID, i18n

_ = i18n.gettext()

class Core :
	"""
	- Sat, 22 Jan 2005 23:47:47 +0900
		At first, to serialize the data of bookmarks, i used 'xmlrpclib'
		modules, but xmlrpclib does not handle the object except tuple
		types, so i found 'marshal' module to loads and dumps the data.

	DATA type
	------------
		eggs : eggs[uid] = (value, label, type)
		list :
		[
			[e0,
				[[e00,], [e01,], [e02,], [e03,]],
			],
			[e1,],
			[e2,
				[[e20,], [e21,], [e22,
					[[e220,], [e221,], [e222,],]
				],
			],
		]

		* uid is made by UID.get()

	"""

	GCONF_CLIENT = None

	DATA = { \
		"list" : list(), \
		"egg" : dict(), \
	}

	def __init__ (self) :

		self.GCONF_CLIENT = GConf.Client()
		self.GCONF_CLIENT.add_dir(Conf.LOCAL_ROOT)

		self.__load()
#		self.DATA["egg"] = { \
#			"VEEdkfajls" : [None, "VEE", "category"], \
#			"ETCdfkjasl" : [None, "ETC", "category"], \
#			"dkfjalsd" : ["/bebop/", "bebop", "terminal"], \
#			"aaadkfjd" : ["/home/spike/.bashrc", ".bashrc", "vim"], \
#			"bbdkfasj" : ["/bebop/Downloads", "Downloads", "terminal"], \
#			"ccfdkfjs" : \
#	["/home/spike/system/lib/vee.run/lib/python2.3/site-packages/vee.py", "vee.py", "gvim"], \
#		}
#	
#		self.DATA["list"] = [ \
#			["VEEdkfajls", \
#				[["aaadkfjd", []], ["ccfdkfjs", []],], \
#			], \
#			["ETCdfkjasl", \
#				[["dkfjalsd", []], ["bbdkfasj", []],], \
#			], \
#		]
#	
#		self.__store()

	def __load (self) :
		try :
			__data0 = \
				self.GCONF_CLIENT.get_value(Conf.SCHEMA["bookmarks"]["key"])
			__data = marshal.loads(base64.decodestring(__data0))
		except :
			return

		# load data and parse it as python value.
		self.DATA = __data

	def __store (self) :
		__data = base64.encodestring(marshal.dumps(self.DATA))

		self.GCONF_CLIENT.set_value( \
			"string", \
			Conf.SCHEMA["bookmarks"]["key"], \
			__data \
		)

	###################################################
	# ACTIONS
	def get_value (self, path) :

		# another method.
		#y = list()
		#for i in range(len(path)) :
		#	y.append(path[i])
		#	if i % 2 : y.append(1)
		#
		#__k = self.DATA["list"]
		#for i in y :
		#	__k = __k[i]
		#
		#return __k

		__child = self.DATA["list"]
		for i in path :
			if len(__child) <= i :
				raise IndexError, "%s does not exists." % i

			__a = __child[i]

			if len(__a) > 1 :
				__parent = __a[0]
				__child = __a[1]
			else :
				__parent = __a[0]
				__child = None

		try :
			__value = self.DATA["egg"][__parent]
		except IndexError :
			return None

		return __value

	def get_from_uid (self, uid) :
		try :
			__value = self.DATA["egg"][uid]
		except IndexError :
			return None

		return __value

	def get_uid_by_path (self, path) :
		try :
			if len(path) == 2 :
				return self.DATA["list"][path[0]][1][path[1]][0]
			elif len(path) == 1 :
				return self.DATA["list"][path[0]][0]
		except IndexError :
			return None

	def __add_by_path (self, path, uid) :

		if len(path) == 1 :
			self.DATA["list"].append([uid, []])
		elif len(path) == 2 :
			if (len(self.DATA["list"]) - 1) < path[0] :
				raise ValueError, "category does not exists."
			else :
				self.DATA["list"][path[0]][1].append([uid, []])
		else :
			raise ValueError, "The length of path must be lower than 2."

		return True

	def add (self, path, type, value, label="", uid="") :
		"""
		returns,
			-1 : duplicated.
			True : added
			False : failed.
		"""

		if type not in \
				("category", "gvim", "vim", "terminal", "external_command", ) :
			raise ValueError, \
"type must be one of 'category', 'gvim', 'vim', 'terminal', 'external_command'."

		if not label.strip() : label = value

		# parse path.
		__values = [value, label, type,]

		# generate uid.
		try :
			__v = self.get_value(path)
		except :
			pass
		else :
			if __v == value : return -1

		if not uid : uid = UID.get()
		self.DATA["egg"][uid] = __values

		if self.__add_by_path(path, uid) :
			self.__store()
			if not uid : return uid
			else : return True
		else :
			return False

	def remove (self, path) :
		uid = self.get_uid_by_path(path)

		if uid :
			if len(path) == 2 :
				del(self.DATA["list"][path[0]][1][path[1]])
			elif len(path) == 1 :
				del(self.DATA["list"][path[0]])
			else :
				raise ValueError, "The depth of path must be lower than 2."
		else :
			raise ValueError, "%s does not exists." % str(path)

		del(self.DATA["egg"][uid])
		self.__store()

	def edit (self, path, **kwargs) :
		for __key, value in kwargs.items() :
			if __key not in \
					("type", "value", "label", ) :
				raise ValueError, \
"The key of argument must be one of 'type', 'value', 'label'."

		__uid = self.get_uid_by_path(path)
		if not __uid : return False

		if kwargs.has_key("value") :
			self.DATA["egg"][__uid][0] = kwargs["value"]
		if kwargs.has_key("label") :
			self.DATA["egg"][__uid][1] = kwargs["label"]
		if kwargs.has_key("type") :
			self.DATA["egg"][__uid][2] = kwargs["type"]

		self.__store()

		return True

	###################################################
	# misc

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "Panel.py"

==> vee_2.2-1.tar.gz: vee-2.2/src/Bookmarks.py <==
# -*- coding: utf-8 -*-
#    Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#    http://vee.kldp.net
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, marshal, time, base64
import Conf, GConf, UID, i18n

_ = i18n.gettext()

class Core :
	"""
	- Sat, 22 Jan 2005 23:47:47 +0900
		At first, to serialize the data of bookmarks, i used 'xmlrpclib'
		modules, but xmlrpclib does not handle the object except tuple
		types, so i found 'marshal' module to loads and dumps the data.

	DATA type
	------------
		eggs : eggs[uid] = (value, label, type)
		list :
		[
			[e0,
				[[e00,], [e01,], [e02,], [e03,]],
			],
			[e1,],
			[e2,
				[[e20,], [e21,], [e22,
					[[e220,], [e221,], [e222,],]
				],
			],
		]

		* uid is made by UID.get()

	"""

	GCONF_CLIENT = None

	DATA = { \
		"list" : list(), \
		"egg" : dict(), \
	}

	def __init__ (self) :

		self.GCONF_CLIENT = GConf.Client()
		self.GCONF_CLIENT.add_dir(Conf.LOCAL_ROOT)

		self.__load()

	def __load (self) :
		try :
			__data0 = \
				self.GCONF_CLIENT.get_value_by_conf("bookmarks")
			__data = marshal.loads(base64.decodestring(__data0))
		except :
			return

		self.DATA = self.__refresh(__data)
		self.__store()

	def __store (self) :
		__data = base64.encodestring(marshal.dumps(self.DATA))

		self.GCONF_CLIENT.set_value_by_conf("bookmarks", __data)

	def __refresh (self, data) :
		__egg = data["egg"]
		__list = data["list"]

		__egg_new = dict()
		__list_new = list()
		for i in __list :
			__list_sub = list()
			if not __egg.has_key(i[0]) :
				continue
			else :
				__egg_new[i[0]] = __egg[i[0]]
				__list_sub = [i[0], []]

			for j in i[1] :
				if not __egg.has_key(j[0]) :
					continue
				else :
					__egg_new[j[0]] = __egg[j[0]]
					__list_sub[1].append([j[0], []])

			if len(__list_sub) > 0 :
				__list_new.append(__list_sub)

		return {"list": __list_new, "egg": __egg_new}

	###################################################
	# ACTIONS
	def get_value (self, path) :

		__child = self.DATA["list"]
		for i in path :
			if len(__child) <= i :
				raise IndexError, "%s does not exists." % i

			__a = __child[i]

			if len(__a) > 1 :
				__parent = __a[0]
				__child = __a[1]
			else :
				__parent = __a[0]
				__child = None

		try :
			__value = self.DATA["egg"][__parent]
		except IndexError :
			return None

		return __value

	def get_from_uid (self, uid) :
		try :
			__value = self.DATA["egg"][uid]
		except KeyError, IndexError :
			return (None, None, None,)
		else :
			return __value

	def get_uid_by_path (self, path) :
		try :
			if len(path) == 2 :
				return self.DATA["list"][path[0]][1][path[1]][0]
			elif len(path) == 1 :
				return self.DATA["list"][path[0]][0]
		except IndexError :
			return None

	def __add_by_path (self, path, uid) :

		if len(path) == 1 :
			self.DATA["list"].append([uid, []])
		elif len(path) == 2 :
			if (len(self.DATA["list"]) - 1) < path[0] :
				raise ValueError, "category does not exists."
			else :
				self.DATA["list"][path[0]][1].append([uid, []])
		else :
			raise ValueError, "The length of path must be lower than 2."

		return True

	def add (self, path, type, value, label="", uid="") :
		"""
		returns,
			-1 : duplicated.
			True : added
			False : failed.
		"""

		if type not in \
				("category", "gvim", "vim", "terminal", "external_command", ) :
			raise ValueError, \
"type must be one of 'category', 'gvim', 'vim', 'terminal', 'external_command'."

		if not label.strip() : label = value

		# parse path.
		__values = [value, label, type,]

		# generate uid.
		try :
			__v = self.get_value(path)
		except :
			pass
		else :
			if __v == value : return -1

		if not uid : uid = UID.get()
		self.DATA["egg"][uid] = __values

		if self.__add_by_path(path, uid) :
			self.__store()
			if not uid : return uid
			else : return True
		else :
			return False

	def remove (self, path) :
		uid = self.get_uid_by_path(path)

		if uid :
			if len(path) == 2 :
				del(self.DATA["list"][path[0]][1][path[1]])
				del(self.DATA["egg"][uid])
			elif len(path) == 1 :
				# find out sub items uid.
				__o = self.DATA["list"][path[0]]

				__list_uid = list()
				__list_uid.append(__o[0])
				if len(__o[1]) > 0 :
					for i in __o[1] :
						__list_uid.append(i[0])

				for i in __list_uid :
					if self.DATA["egg"].has_key(i) :
						del(self.DATA["egg"][i])

				del(self.DATA["list"][path[0]])

			else :
				raise ValueError, "The depth of path must be lower than 2."
		else :
			raise ValueError, "%s does not exists." % str(path)

		self.__store()

	def edit (self, path, **kwargs) :
		for __key, value in kwargs.items() :
			if __key not in \
					("type", "value", "label", ) :
				raise ValueError, \
"The key of argument must be one of 'type', 'value', 'label'."

		__uid = self.get_uid_by_path(path)
		if not __uid : return False

		if kwargs.has_key("value") :
			self.DATA["egg"][__uid][0] = kwargs["value"]
		if kwargs.has_key("label") :
			self.DATA["egg"][__uid][1] = kwargs["label"]
		if kwargs.has_key("type") :
			self.DATA["egg"][__uid][2] = kwargs["type"]

		self.__store()

		return True

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "Panel.py"
